/* File.....: isbn.prg
 * Author...: Peter Townsend
 * CIS ID...:
 * e-mail...: cephas@tpgi.com.au
 *
 * A function to validate ISBN numbers
 *     ISBNCheck()
 *
 */
/*
 * This is an original work by Peter Townsend and is
 * hereby placed in the public domain.
 */
/*  $DOC$
 *  $FUNCNAME$
 *      ISBNCheck()
 *
 *  $ONELINER$
 *      Validate an ISBN number
 *
 *  $SYNTAX$
 *      ISBNCheck( <cISBN> ) -> lValid
 *
 *  $ARGUMENTS$
 *      cISBN      - the ISBN number to be validated
 *
 *  $RETURNS$
 *      lValid     - is the ISBN number valid?
 *
 *  $DESCRIPTION$
 *      This function validates ISBN numbers using the checksum on the end
 *      of the ISBN number.  It discards non-digit characters (apart from the
 *      checksum, which may be an "X") so it doesn't matter whether spaces or
 *      dashes or anything else is used as a separator, or even if they're
 *      included at all.  It therefore doesn't check for a specific length for
 *      an ISBN number, but does check that exactly 10 valid characters (the
 *      number proper, and the checksum digit) are processed.
 *
 *  $EXAMPLES$
 *      IF ! ISBNCheck( cISBN )
 *          ALERT( "Invalid ISBN number " )
 *      ENDIF
 *
 *  $SEEALSO$
 *
 *  $INCLUDE$
 *
 *  $END$
 */
FUNCTION ISBNCheck( cISBN )
    LOCAL nLength, nCharCntr, cThisChar, nCheckSum
    LOCAL nModVal, nCheckPos, lIsValid
    cISBN     := ALLTRIM( cISBN )
    nCheckPos := 0
    nCheckSum := 0
    nLength   := LEN( cISBN ) - 1
    FOR nCharCntr := 1 TO nLength
        cThisChar := SUBSTR( cISBN, nCharCntr, 1 )
        IF ISDIGIT( cThisChar )
            nCheckPos++
            nCheckSum += ( VAL( cThisChar ) * nCheckPos )
        ENDIF
    NEXT
    IF nCheckPos <> 9
        lIsValid := .F.
    ELSE
        cThisChar := UPPER( RIGHT( cISBN, 1 ) )
        DO CASE
            CASE cThisChar == "X"
                nModVal := 10
            CASE ISDIGIT( cThisChar )
                nModVal := VAL( cThisChar )
            OTHERWISE
                nModVal := -1   // Force an error
        ENDCASE
        lIsValid   := ( MOD( nCheckSum, 11 ) == nModVal )
    ENDIF
RETURN( lIsValid )
